#! /usr/bin/python
#-*- coding: utf-8 -*- 

#from label import Ui_Frame as Label
#from control_spin import Ui_Frame as ControlSpin
#from name import Ui_Frame as Name
#from control_slider import Ui_Frame as ControlSlider
##from control_button import Ui_Form as ControlButton
##from control_progress import Ui_Frame as ControlProgress
