#! /usr/bin/python
#-*- coding: utf-8 -*- 


#from part import Ui_Form as Floatable
#from inside import Ui_Form as Inside
#from inside2 import Ui_Form as Inside2

#from part_bool import Ui_Form as FloatableBool
#from inside_bool import Ui_Form as InsideBool
#from inside2_bool import Ui_Form as Inside2Bool
